@@ -59,7 +59,7 @@ static void
 wibox_need_update(wibox_t *wibox)
 {
     wibox->need_update = true;
-    wibox->mouse_over = NULL;
+    wibox_clear_mouse_over(wibox);
 }
 
 static int
@@ -166,12 +166,13 @@ wibox_init(wibox_t *w, int phys_screen)
                       w->geometry.x, w->geometry.y,
                       w->geometry.width, w->geometry.height,
                       w->border_width, XCB_COPY_FROM_PARENT, s->root_visual,
-                      XCB_CW_BACK_PIXMAP | XCB_CW_BORDER_PIXEL
+                      XCB_CW_BACK_PIXEL | XCB_CW_BORDER_PIXEL | XCB_CW_BIT_GRAVITY
                       | XCB_CW_OVERRIDE_REDIRECT | XCB_CW_EVENT_MASK,
                       (const uint32_t [])
                       {
-                          XCB_BACK_PIXMAP_PARENT_RELATIVE,
+                          w->ctx.bg.pixel,
                           w->border_color.pixel,
+                          XCB_GRAVITY_NORTH_WEST,
                           1,
                           XCB_EVENT_MASK_SUBSTRUCTURE_REDIRECT
                           | XCB_EVENT_MASK_SUBSTRUCTURE_NOTIFY | XCB_EVENT_MASK_ENTER_WINDOW
@@ -335,8 +336,9 @@ static void
 wibox_set_border_color(lua_State *L, int udx, const xcolor_t *color)
 {
     wibox_t *w = luaA_checkudata(L, udx, &wibox_class);
-    xcb_change_window_attributes(globalconf.connection, w->window,
-                                 XCB_CW_BORDER_PIXEL, &color->pixel);
+    if (w->window != XCB_NONE)
+        xcb_change_window_attributes(globalconf.connection, w->window,
+                                     XCB_CW_BORDER_PIXEL, &color->pixel);
     w->border_color = *color;
     luaA_object_emit_signal(L, udx, "property::border_color", 0);
 }
@@ -617,6 +619,19 @@ wibox_refresh(void)
     }
 }
 
+/** Clear the wibox' mouse_over pointer.
+ * \param wibox The wibox.
+ */
+void
+wibox_clear_mouse_over(wibox_t *wibox)
+{
+    if (wibox->mouse_over)
+    {
+        luaA_object_unref(globalconf.L, wibox->mouse_over);
+        wibox->mouse_over = NULL;
+    }
+}
+
 /** Set a wibox visible or not.
  * \param L The Lua VM state.
  * \param udx The wibox.
@@ -629,7 +644,7 @@ wibox_set_visible(lua_State *L, int udx, bool v)
     if(v != wibox->visible)
     {
         wibox->visible = v;
-        wibox->mouse_over = NULL;
+        wibox_clear_mouse_over(wibox);
 
         if(wibox->screen)
         {
@@ -702,7 +717,7 @@ wibox_detach(lua_State *L, int udx)
         /* restore visibility */
         wibox->visible = v;
 
-        wibox->mouse_over = NULL;
+        wibox_clear_mouse_over(wibox);
 
         wibox_wipe(wibox);
 
@@ -1047,7 +1062,18 @@ luaA_wibox_set_bg(lua_State *L, wibox_t *wibox)
     size_t len;
     const char *buf = luaL_checklstring(L, -1, &len);
     if(xcolor_init_reply(xcolor_init_unchecked(&wibox->ctx.bg, buf, len)))
+    {
+        uint32_t mask = XCB_CW_BACK_PIXEL;
+        uint32_t values[] = { wibox->ctx.bg.pixel };
+
         wibox->need_update = true;
+
+        if (wibox->window != XCB_NONE)
+            xcb_change_window_attributes(globalconf.connection,
+                                         wibox->window,
+                                         mask,
+                                         values);
+    }
     luaA_object_emit_signal(L, -3, "property::bg", 0);
     return 0;
 }
@@ -1449,11 +1475,12 @@ static int
 luaA_wibox_set_border_width(lua_State *L, wibox_t *wibox)
 {
     wibox_t *w = luaA_checkudata(L, -3, &wibox_class);
-    uint32_t border_width = luaL_checknumber(L, -1);
-    if(border_width != w->border_width)
+    int32_t border_width = luaL_checknumber(L, -1);
+    if(border_width != w->border_width && border_width >= 0)
     {
-        xcb_configure_window(globalconf.connection, w->window, XCB_CONFIG_WINDOW_BORDER_WIDTH,
-                             &border_width);
+        if (w->window != XCB_NONE)
+            xcb_configure_window(globalconf.connection, w->window, XCB_CONFIG_WINDOW_BORDER_WIDTH,
+                                 (uint32_t[]) { border_width });
         w->border_width = border_width;
         /* Need update if transparent background */
         wibox_need_update(w);
